@@ -20,6 +20,12 @@ from msal.telemetry import CLIENT_CURRENT_TELEMETRY, CLIENT_LAST_TELEMETRY
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG)
 
+_OIDC_DISCOVERY = "msal.authority.tenant_discovery"
+_OIDC_DISCOVERY_MOCK = Mock(return_value={
+    "authorization_endpoint": "https://contoso.com/placeholder",
+    "token_endpoint": "https://contoso.com/placeholder",
+})
+
 
 class TestHelperExtractCerts(unittest.TestCase):  # It is used by SNI scenario
 
@@ -58,10 +64,9 @@ class TestBytesConversion(unittest.TestCase):
 
 class TestClientApplicationAcquireTokenSilentErrorBehaviors(unittest.TestCase):
 
+    @patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
     def setUp(self):
         self.authority_url = "https://login.microsoftonline.com/common"
-        self.authority = msal.authority.Authority(
-            self.authority_url, MinimalHttpClient())
         self.scopes = ["s1", "s2"]
         self.uid = "my_uid"
         self.utid = "my_utid"
@@ -116,12 +121,11 @@ class TestClientApplicationAcquireTokenSilentErrorBehaviors(unittest.TestCase):
         self.assertEqual("", result.get("classification"))
 
 
+@patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
 class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
 
     def setUp(self):
         self.authority_url = "https://login.microsoftonline.com/common"
-        self.authority = msal.authority.Authority(
-            self.authority_url, MinimalHttpClient())
         self.scopes = ["s1", "s2"]
         self.uid = "my_uid"
         self.utid = "my_utid"
@@ -148,9 +152,9 @@ class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
             self.assertEqual(self.frt, data.get("refresh_token"), "Should attempt the FRT")
             return MinimalResponse(status_code=400, text=error_response)
         app._acquire_token_silent_by_finding_rt_belongs_to_me_or_my_family(
-            self.authority, self.scopes, self.account, post=tester)
-        self.assertNotEqual([], app.token_cache.find(
-            msal.TokenCache.CredentialType.REFRESH_TOKEN, query={"secret": self.frt}),
+            app.authority, self.scopes, self.account, post=tester)
+        self.assertIsNotNone(next(app.token_cache.search(
+            msal.TokenCache.CredentialType.REFRESH_TOKEN, query={"secret": self.frt}), None),
             "The FRT should not be removed from the cache")
 
     def test_known_orphan_app_will_skip_frt_and_only_use_its_own_rt(self):
@@ -168,7 +172,7 @@ class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
             self.assertEqual(rt, data.get("refresh_token"), "Should attempt the RT")
             return MinimalResponse(status_code=200, text='{}')
         app._acquire_token_silent_by_finding_rt_belongs_to_me_or_my_family(
-            self.authority, self.scopes, self.account, post=tester)
+            app.authority, self.scopes, self.account, post=tester)
 
     def test_unknown_family_app_will_attempt_frt_and_join_family(self):
         def tester(url, data=None, **kwargs):
@@ -180,14 +184,14 @@ class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
         app = ClientApplication(
             "unknown_family_app", authority=self.authority_url, token_cache=self.cache)
         at = app._acquire_token_silent_by_finding_rt_belongs_to_me_or_my_family(
-            self.authority, self.scopes, self.account, post=tester)
+            app.authority, self.scopes, self.account, post=tester)
         logger.debug("%s.cache = %s", self.id(), self.cache.serialize())
         self.assertEqual("at", at.get("access_token"), "New app should get a new AT")
-        app_metadata = app.token_cache.find(
+        app_metadata = next(app.token_cache.search(
             msal.TokenCache.CredentialType.APP_METADATA,
-            query={"client_id": app.client_id})
-        self.assertNotEqual([], app_metadata, "Should record new app's metadata")
-        self.assertEqual("1", app_metadata[0].get("family_id"),
+            query={"client_id": app.client_id}), None)
+        self.assertIsNotNone(app_metadata, "Should record new app's metadata")
+        self.assertEqual("1", app_metadata.get("family_id"),
             "The new family app should be recorded as in the same family")
     # Known family app will simply use FRT, which is largely the same as this one
 
@@ -202,7 +206,7 @@ class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
         app = ClientApplication(
             "preexisting_family_app", authority=self.authority_url, token_cache=self.cache)
         resp = app._acquire_token_silent_by_finding_rt_belongs_to_me_or_my_family(
-            self.authority, self.scopes, self.account, post=tester)
+            app.authority, self.scopes, self.account, post=tester)
         logger.debug("%s.cache = %s", self.id(), self.cache.serialize())
         self.assertEqual(json.loads(error_response), resp, "Error raised will be returned")
 
@@ -214,30 +218,30 @@ class TestClientApplicationAcquireTokenSilentFociBehaviors(unittest.TestCase):
         account = app.get_accounts()[0]
         mine = {"home_account_id": account["home_account_id"]}
 
-        self.assertNotEqual([], self.cache.find(
-            self.cache.CredentialType.ACCESS_TOKEN, query=mine))
-        self.assertNotEqual([], self.cache.find(
-            self.cache.CredentialType.REFRESH_TOKEN, query=mine))
-        self.assertNotEqual([], self.cache.find(
-            self.cache.CredentialType.ID_TOKEN, query=mine))
-        self.assertNotEqual([], self.cache.find(
-            self.cache.CredentialType.ACCOUNT, query=mine))
+        self.assertIsNotNone(next(self.cache.search(
+            self.cache.CredentialType.ACCESS_TOKEN, query=mine), None))
+        self.assertIsNotNone(next(self.cache.search(
+            self.cache.CredentialType.REFRESH_TOKEN, query=mine), None))
+        self.assertIsNotNone(next(self.cache.search(
+            self.cache.CredentialType.ID_TOKEN, query=mine), None))
+        self.assertIsNotNone(next(self.cache.search(
+            self.cache.CredentialType.ACCOUNT, query=mine), None))
 
         app.remove_account(account)
 
-        self.assertEqual([], self.cache.find(
-            self.cache.CredentialType.ACCESS_TOKEN, query=mine))
-        self.assertEqual([], self.cache.find(
-            self.cache.CredentialType.REFRESH_TOKEN, query=mine))
-        self.assertEqual([], self.cache.find(
-            self.cache.CredentialType.ID_TOKEN, query=mine))
-        self.assertEqual([], self.cache.find(
-            self.cache.CredentialType.ACCOUNT, query=mine))
+        self.assertIsNone(next(self.cache.search(
+            self.cache.CredentialType.ACCESS_TOKEN, query=mine), None))
+        self.assertIsNone(next(self.cache.search(
+            self.cache.CredentialType.REFRESH_TOKEN, query=mine), None))
+        self.assertIsNone(next(self.cache.search(
+            self.cache.CredentialType.ID_TOKEN, query=mine), None))
+        self.assertIsNone(next(self.cache.search(
+            self.cache.CredentialType.ACCOUNT, query=mine), None))
 
 
 class TestClientApplicationForAuthorityMigration(unittest.TestCase):
 
-    @classmethod
+    # Chose to not mock oidc discovery, because AuthorityMigration might rely on real data
     def setUp(self):
         self.environment_in_cache = "sts.windows.net"
         self.authority_url_in_app = "https://login.microsoftonline.com/common"
@@ -340,6 +344,7 @@ class TestApplicationForRefreshInBehaviors(unittest.TestCase):
     account = {"home_account_id": "{}.{}".format(uid, utid)}
     rt = "this is a rt"
     client_id = "my_app"
+    soon = 60  # application.py considers tokens within 5 minutes as expired
 
     @classmethod
     def setUpClass(cls):  # Initialization at runtime, not interpret-time
@@ -414,7 +419,8 @@ class TestApplicationForRefreshInBehaviors(unittest.TestCase):
 
     def test_expired_token_and_unavailable_aad_should_return_error(self):
         # a.k.a. Attempt refresh expired token when AAD unavailable
-        self.populate_cache(access_token="expired at", expires_in=-1, refresh_in=-900)
+        self.populate_cache(
+            access_token="expired at", expires_in=self.soon, refresh_in=-900)
         error = "something went wrong"
         def mock_post(url, headers=None, *args, **kwargs):
             self.assertEqual("4|84,3|", (headers or {}).get(CLIENT_CURRENT_TELEMETRY))
@@ -425,7 +431,8 @@ class TestApplicationForRefreshInBehaviors(unittest.TestCase):
 
     def test_expired_token_and_available_aad_should_return_new_token(self):
         # a.k.a. Attempt refresh expired token when AAD available
-        self.populate_cache(access_token="expired at", expires_in=-1, refresh_in=-900)
+        self.populate_cache(
+            access_token="expired at", expires_in=self.soon, refresh_in=-900)
         new_access_token = "new AT"
         new_refresh_in = 123
         def mock_post(url, headers=None, *args, **kwargs):
@@ -441,6 +448,7 @@ class TestApplicationForRefreshInBehaviors(unittest.TestCase):
         self.assertRefreshOn(result, new_refresh_in)
 
 
+# TODO Patching oidc discovery ends up failing. But we plan to remove offline telemetry anyway.
 class TestTelemetryMaintainingOfflineState(unittest.TestCase):
     authority_url = "https://login.microsoftonline.com/common"
     scopes = ["s1", "s2"]
@@ -521,6 +529,7 @@ class TestTelemetryMaintainingOfflineState(unittest.TestCase):
 
 class TestTelemetryOnClientApplication(unittest.TestCase):
     @classmethod
+    @patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
     def setUpClass(cls):  # Initialization at runtime, not interpret-time
         cls.app = ClientApplication(
             "client_id", authority="https://login.microsoftonline.com/common")
@@ -549,6 +558,7 @@ class TestTelemetryOnClientApplication(unittest.TestCase):
 
 class TestTelemetryOnPublicClientApplication(unittest.TestCase):
     @classmethod
+    @patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
     def setUpClass(cls):  # Initialization at runtime, not interpret-time
         cls.app = PublicClientApplication(
             "client_id", authority="https://login.microsoftonline.com/common")
@@ -578,6 +588,7 @@ class TestTelemetryOnPublicClientApplication(unittest.TestCase):
 
 class TestTelemetryOnConfidentialClientApplication(unittest.TestCase):
     @classmethod
+    @patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
     def setUpClass(cls):  # Initialization at runtime, not interpret-time
         cls.app = ConfidentialClientApplication(
             "client_id", client_credential="secret",
@@ -623,6 +634,7 @@ class TestTelemetryOnConfidentialClientApplication(unittest.TestCase):
         self.assertEqual(at, result.get("access_token"))
 
 
+@patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
 class TestClientApplicationWillGroupAccounts(unittest.TestCase):
     def test_get_accounts(self):
         client_id = "my_app"
@@ -675,29 +687,38 @@ class TestClientCredentialGrant(unittest.TestCase):
         with self.assertWarns(DeprecationWarning):
             app.acquire_token_for_client(["scope"], post=mock_post)
 
+    @patch(_OIDC_DISCOVERY, new=Mock(return_value={
+        "authorization_endpoint": "https://contoso.com/common",
+        "token_endpoint": "https://contoso.com/common",
+        }))
     def test_common_authority_should_emit_warning(self):
         self._test_certain_authority_should_emit_warning(
             authority="https://login.microsoftonline.com/common")
 
+    @patch(_OIDC_DISCOVERY, new=Mock(return_value={
+        "authorization_endpoint": "https://contoso.com/organizations",
+        "token_endpoint": "https://contoso.com/organizations",
+        }))
     def test_organizations_authority_should_emit_warning(self):
         self._test_certain_authority_should_emit_warning(
             authority="https://login.microsoftonline.com/organizations")
 
 
+@patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
 class TestRemoveTokensForClient(unittest.TestCase):
     def test_remove_tokens_for_client_should_remove_client_tokens_only(self):
         at_for_user = "AT for user"
         cca = msal.ConfidentialClientApplication(
             "client_id", client_credential="secret",
             authority="https://login.microsoftonline.com/microsoft.onmicrosoft.com")
-        self.assertEqual(
-            0, len(cca.token_cache.find(msal.TokenCache.CredentialType.ACCESS_TOKEN)))
+        self.assertIsNone(next(cca.token_cache.search(
+            msal.TokenCache.CredentialType.ACCESS_TOKEN), None))
         cca.acquire_token_for_client(
             ["scope"],
             post=lambda url, **kwargs: MinimalResponse(
                 status_code=200, text=json.dumps({"access_token": "AT for client"})))
-        self.assertEqual(
-            1, len(cca.token_cache.find(msal.TokenCache.CredentialType.ACCESS_TOKEN)))
+        self.assertEqual(1, len(list(cca.token_cache.search(
+            msal.TokenCache.CredentialType.ACCESS_TOKEN))))
         cca.acquire_token_by_username_password(
             "johndoe", "password", ["scope"],
             post=lambda url, **kwargs: MinimalResponse(
@@ -705,14 +726,16 @@ class TestRemoveTokensForClient(unittest.TestCase):
                     access_token=at_for_user, expires_in=3600,
                     uid="uid", utid="utid",  # This populates home_account_id
                     ))))
-        self.assertEqual(
-            2, len(cca.token_cache.find(msal.TokenCache.CredentialType.ACCESS_TOKEN)))
+        self.assertEqual(2, len(list(cca.token_cache.search(
+            msal.TokenCache.CredentialType.ACCESS_TOKEN))))
         cca.remove_tokens_for_client()
-        remaining_tokens = cca.token_cache.find(msal.TokenCache.CredentialType.ACCESS_TOKEN)
+        remaining_tokens = list(cca.token_cache.search(
+            msal.TokenCache.CredentialType.ACCESS_TOKEN))
         self.assertEqual(1, len(remaining_tokens))
         self.assertEqual(at_for_user, remaining_tokens[0].get("secret"))
 
 
+@patch(_OIDC_DISCOVERY, new=_OIDC_DISCOVERY_MOCK)
 class TestScopeDecoration(unittest.TestCase):
     def _test_client_id_should_be_a_valid_scope(self, client_id, other_scopes):
         # B2C needs this https://learn.microsoft.com/en-us/azure/active-directory-b2c/access-tokens#openid-connect-scopes
@@ -733,8 +756,49 @@ class TestScopeDecoration(unittest.TestCase):
     "authorization_endpoint": "https://contoso.com/placeholder",
     "token_endpoint": "https://contoso.com/placeholder",
     }))
-@patch("msal.application._init_broker", new=Mock())  # Allow testing without pymsalruntime
-class TestBrokerFallback(unittest.TestCase):
+class TestMsalBehaviorWithoutPyMsalRuntimeOrBroker(unittest.TestCase):
+
+    @patch("msal.application._init_broker", new=Mock(side_effect=ImportError(
+        "PyMsalRuntime not installed"
+    )))
+    def test_broker_should_be_disabled_by_default(self):
+        app = msal.PublicClientApplication(
+            "client_id",
+            authority="https://login.microsoftonline.com/common",
+            )
+        self.assertFalse(app._enable_broker)
+
+    @patch("msal.application._init_broker", new=Mock(side_effect=ImportError(
+        "PyMsalRuntime not installed"
+    )))
+    def test_opt_in_should_error_out_when_pymsalruntime_not_installed(self):
+        """Because it is actionable to app developer to add dependency declaration"""
+        with self.assertRaises(ImportError):
+            app = msal.PublicClientApplication(
+                "client_id",
+                authority="https://login.microsoftonline.com/common",
+                enable_broker_on_mac=True,
+                )
+
+    @patch("msal.application._init_broker", new=Mock(side_effect=RuntimeError(
+        "PyMsalRuntime raises RuntimeError when broker initialization failed"
+    )))
+    def test_should_fallback_when_pymsalruntime_failed_to_initialize_broker(self):
+        app = msal.PublicClientApplication(
+            "client_id",
+            authority="https://login.microsoftonline.com/common",
+            enable_broker_on_mac=True,
+            )
+        self.assertFalse(app._enable_broker)
+
+
+@patch("sys.platform", new="darwin")  # Pretend running on Mac.
+@patch("msal.authority.tenant_discovery", new=Mock(return_value={
+    "authorization_endpoint": "https://contoso.com/placeholder",
+    "token_endpoint": "https://contoso.com/placeholder",
+    }))
+@patch("msal.application._init_broker", new=Mock())  # Pretend pymsalruntime installed and working
+class TestBrokerFallbackWithDifferentAuthorities(unittest.TestCase):
 
     def test_broker_should_be_disabled_by_default(self):
         app = msal.PublicClientApplication(
@@ -787,3 +851,27 @@ class TestBrokerFallback(unittest.TestCase):
             )
         self.assertFalse(app._enable_broker)
 
+    def test_app_did_not_register_redirect_uri_should_error_out(self):
+        """Because it is actionable to app developer to add redirect URI"""
+        app = msal.PublicClientApplication(
+            "client_id",
+            authority="https://login.microsoftonline.com/common",
+            enable_broker_on_mac=True,
+            )
+        self.assertTrue(app._enable_broker)
+        with patch.object(
+            # Note: We tried @patch("msal.broker.foo", ...) but it ended up with
+            # "module msal does not have attribute broker"
+            app, "_acquire_token_interactive_via_broker", return_value={
+                "error": "broker_error",
+                "error_description":
+                    "(pii).  "  # pymsalruntime no longer surfaces AADSTS error,
+                                # So MSAL Python can't raise RedirectUriError.
+                    "Status: Response_Status.Status_ApiContractViolation, "
+                    "Error code: 3399614473, Tag 557973642",
+            }):
+            result = app.acquire_token_interactive(
+                ["scope"],
+                parent_window_handle=app.CONSOLE_WINDOW_HANDLE,
+                )
+            self.assertEqual(result.get("error"), "broker_error")
