@@ -193,6 +193,8 @@ class E2eTestCase(unittest.TestCase):
                 http_client=http_client or MinimalHttpClient(),
                 enable_broker_on_windows=_PYMSALRUNTIME_INSTALLED,
                 enable_broker_on_mac=_PYMSALRUNTIME_INSTALLED,
+                enable_broker_on_linux=_PYMSALRUNTIME_INSTALLED,
+                enable_broker_on_wsl=_PYMSALRUNTIME_INSTALLED,
                 )
 
     def _test_username_password(self,
@@ -839,12 +841,14 @@ class AtPopWithExternalKeyTestCase(PopWithExternalKeyTestCase):
 
 
 class WorldWideTestCase(LabBasedTestCase):
+    _ADFS_LABS_DECOMMISSIONED = "ADFS labs were decommissioned since July 2025 until further notice"
 
     def test_aad_managed_user(self):  # Pure cloud
         config = self.get_lab_user(usertype="cloud")
         config["password"] = self.get_lab_user_secret(config["lab_name"])
         self._test_username_password(**config)
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_adfs4_fed_user(self):
         config = self.get_lab_user(usertype="federated", federationProvider="ADFSv4")
         config["password"] = self.get_lab_user_secret(config["lab_name"])
@@ -862,6 +866,7 @@ class WorldWideTestCase(LabBasedTestCase):
         config["password"] = self.get_lab_user_secret(config["lab_name"])
         self._test_username_password(**config)
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_adfs2019_fed_user(self):
         try:
             config = self.get_lab_user(usertype="federated", federationProvider="ADFSv2019")
@@ -890,6 +895,7 @@ class WorldWideTestCase(LabBasedTestCase):
             prompt="select_account",  # In MSAL Python, this resets login_hint
             ))
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_ropc_adfs2019_onprem(self):
         # Configuration is derived from https://github.com/AzureAD/microsoft-authentication-library-for-dotnet/blob/4.7.0/tests/Microsoft.Identity.Test.Common/TestConstants.cs#L250-L259
         config = self.get_lab_user(usertype="onprem", federationProvider="ADFSv2019")
@@ -898,6 +904,7 @@ class WorldWideTestCase(LabBasedTestCase):
         config["password"] = self.get_lab_user_secret(config["lab_name"])
         self._test_username_password(**config)
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_adfs2019_onprem_acquire_token_by_auth_code(self):
         """When prompted, you can manually login using this account:
 
@@ -911,6 +918,7 @@ class WorldWideTestCase(LabBasedTestCase):
         config["port"] = 8080
         self._test_acquire_token_by_auth_code(**config)
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_adfs2019_onprem_acquire_token_by_auth_code_flow(self):
         config = self.get_lab_user(usertype="onprem", federationProvider="ADFSv2019")
         self._test_acquire_token_by_auth_code_flow(**dict(
@@ -920,6 +928,7 @@ class WorldWideTestCase(LabBasedTestCase):
             port=8080,
             ))
 
+    @unittest.skip(_ADFS_LABS_DECOMMISSIONED)
     def test_adfs2019_onprem_acquire_token_interactive(self):
         config = self.get_lab_user(usertype="onprem", federationProvider="ADFSv2019")
         self._test_acquire_token_interactive(**dict(
@@ -934,10 +943,10 @@ class WorldWideTestCase(LabBasedTestCase):
         "Need LAB_OBO_CLIENT_SECRET from https://aka.ms/GetLabSecret?Secret=TodoListServiceV2-OBO")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_CONFIDENTIAL_CLIENT_ID"),
-        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_PUBLIC_CLIENT_ID"),
-        "Need LAB_OBO_PUBLIC_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_PUBLIC_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     def test_acquire_token_obo(self):
         config = self.get_lab_user(usertype="cloud")
 
@@ -998,7 +1007,7 @@ class WorldWideTestCase(LabBasedTestCase):
         "Need LAB_OBO_CLIENT_SECRET from https://aka.ms/GetLabSecret?Secret=TodoListServiceV2-OBO")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_CONFIDENTIAL_CLIENT_ID"),
-        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     def test_confidential_client_acquire_token_by_username_password(self):
         # This approach won't work:
         #       config = self.get_lab_user(usertype="cloud", publicClient="no")
@@ -1215,10 +1224,10 @@ class WorldWideRegionalEndpointTestCase(LabBasedTestCase):
         "Need LAB_OBO_CLIENT_SECRET from https://aka.ms/GetLabSecret?Secret=TodoListServiceV2-OBO")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_CONFIDENTIAL_CLIENT_ID"),
-        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_PUBLIC_CLIENT_ID"),
-        "Need LAB_OBO_PUBLIC_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_PUBLIC_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     def test_cca_obo_should_bypass_regional_endpoint_therefore_still_work(self):
         """We test OBO because it is implemented in sub class ConfidentialClientApplication"""
         config = self.get_lab_user(usertype="cloud")
@@ -1246,7 +1255,7 @@ class WorldWideRegionalEndpointTestCase(LabBasedTestCase):
         "Need LAB_OBO_CLIENT_SECRET from https://aka.ms/GetLabSecret?Secret=TodoListServiceV2-OBO")
     @unittest.skipUnless(
         os.getenv("LAB_OBO_CONFIDENTIAL_CLIENT_ID"),
-        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://docs.msidlab.com/flows/onbehalfofflow.html")
+        "Need LAB_OBO_CONFIDENTIAL_CLIENT_ID from https://identitydivision.visualstudio.com/Engineering/_git/IDLABS?path=/src/DocFX/LabDocs/flows/onbehalfofflow.md&_a=preview")
     def test_cca_ropc_should_bypass_regional_endpoint_therefore_still_work(self):
         """We test ROPC because it is implemented in base class ClientApplication"""
         config = self.get_lab_user(usertype="cloud")
